// Copyright 2005 and onwards, Google
// Author:      Aaron Boodman
// Modified By: Glen Murphy
// Modified By: Liz Looney
//
// FFLoadGraph bootstrapper. All the files required by FFLoadGraph are loaded
// into this service's JS context and run here. This gives us a nice, stable,
// and clean area to play in which cannot be polluted by other extensions and
// which lasts the entire lifetime of Firefox.
//
// This bootstrapper works for Firefox 1.5.x. It uses the __LOCATION__ variable,
// which doesn't exist in Firefox 1.0.x.
//
// The file registers one XPCOM component:
// "@google.com/ffloadgraph/application;1"
//
// Direct Dependencies:
//                                     - Components, __LOCATION__
//   ../lib/firefox/jsmodule.js        - G_JSModule
//   ../lib/ffloadgraph-application.js - FFLGApplication

// Firefox scripts assume that these Cc and Ci are defined.
var Cc = Components.classes;
var Ci = Components.interfaces;

// G_DebugService relies on this flag existing in outside code.
var G_GDEBUG = false;

// g_jsModule will be created in function fflgStartUp, after
// sub-scripts are loaded.
var g_jsModule = null;

// fflgApplication will be created in function fflgStartUp, after
// sub-scripts are loaded.
var fflgApplication = null;

fflgStartUp();

//..............................................................................

/**
 * NSGetModule is the entry point.
 */
function NSGetModule() {
  // Return the module.
  return g_jsModule;
}

//..............................................................................

/**
 * fflgStartUp loads all the sub-scripts, creates the G_JSModule and the
 * FFLGApplication, and registers the application.
 */
function fflgStartUp() {
  // Load sub-scripts into this context.
  var subScriptLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].
                        getService(Ci.mozIJSSubScriptLoader);

  var url;

  // In opt mode, we only load the compiled js.
  var loadedCompiledJavaScript = false;
  try {
    url = fflgGetLibUrl("ffloadgraph.js");
    subScriptLoader.loadSubScript(url);
    loadedCompiledJavaScript = true;
  } catch (e) {
    // An exception is expected if the compiled javascript doesn't exist.
  }

  if (!loadedCompiledJavaScript) {
    // In debug mode, we load the manifest to see which files need to be
    // loaded.
    var libs = fflgReadManifest("ffloadgraph.MF");

    for (var i = 0; i < libs.length; ++i) {
      url = fflgGetLibUrl(libs[i]);
      subScriptLoader.loadSubScript(url);
    }
  }

  // Now that we've loaded the sub-scripts, we can create the G_JSModule.
  g_jsModule = new G_JSModule();

  // Create the application.
  fflgApplication = new FFLGApplication();

  // Register the application.
  g_jsModule.registerObject("{c9a7bfd7-8785-45dc-8795-d2782baa0801}",
                            "@google.com/ffloadgraph/application;1",
                            "FFLGApplication",
                            fflgApplication,
                            ["http-startup-category"]);
}


/**
 * fflgGetLibFile returns a file for the given physical path relative to the
 * lib/ folder.
 *
 * @param path string
 * @return nsIFile
 */
function fflgGetLibFile(path) {
  var file = __LOCATION__.parent.parent;
  file.append("lib");

  var parts = path.split("/");
  for (var i = 0; i < parts.length; ++i) {
    file.append(parts[i]);
  }

  if (!file.exists()) {
    throw new Error("Specified file {" + file.path + "} does not exist.");
  }

  return file;
}


/**
 * fflgGetLibUrl returns a file:// URL spec for the given physical path relative
 * to the lib/ folder.
 *
 * @param path string
 * @return string url spec
 */
function fflgGetLibUrl(path) {
  var file = fflgGetLibFile(path);
  return Cc["@mozilla.org/network/protocol;1?name=file"].
         getService(Ci.nsIFileProtocolHandler).
         getURLSpecFromFile(file);
}


/**
 * fflgReadManifest reads the given manifest file and returns an array where
 * each element is a line from the manifest file.
 *
 * @param manifestFileName string
 * @return array of strings
 */
function fflgReadManifest(manifestFileName) {
  var file = fflgGetLibFile(manifestFileName);

  var fs = Cc["@mozilla.org/network/file-input-stream;1"].
           createInstance(Ci.nsIFileInputStream);
  fs.init(file, 0x01, /* read only */ 0444, 0);

  fs.QueryInterface(Ci.nsILineInputStream);
  var entries = [];
  do {
    var line = {value : null};
    var more = fs.readLine(line);
    if (line.value !== null) {
      entries[entries.length] = line.value;
    }
  } while (more);

  fs.close();

  return entries;
}


//..............................................................................
